{
TColorBox :   this is a color combobox

Author: Giuseppe Madaffari
Copyright:  2001 by Giuseppe Madaffari. All rights reserved.
Created: 10/10/2000
Modified: 23/10/2001
Version: 1.0
License: you can freely use and distribute the included code
         for any purpouse, but you cannot remove this copyright
         notice. Send me any comment and update, they are really
         appreciated.
Contact: gmadaffari@infomedia.it
         http://www.programmers.net/artic/Madaffari/
         http://web.tiscali.it/gmadaffari/
}
unit QColorBox;

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls;

type
  TColorBox = class(TCustomComboBox)
  private
    FCustomColor: TColor;
    FShowCustomColor: Boolean;
    function GetSelectedColor: TColor;
    procedure SetShowCustomColor(const Value: Boolean);
    procedure SetCustomColor(const Value: TColor);
  protected
    procedure Change; override;
    function DrawItem(Index: Integer; Rect: TRect;
      State: TOwnerDrawState): Boolean; override;
    procedure ColorCallBack(const ColorName: string);
    procedure PopulateItems;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Anchors;
    property Color default clBase;
    property Constraints;
    property CustomColor: TColor read FCustomColor write SetCustomColor;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property ItemHeight;
    property ItemIndex;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint default False;
    property SelectedColor: TColor read GetSelectedColor;
    property ShowCustomColor: Boolean read FShowCustomColor write SetShowCustomColor;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnCloseUp;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnHighlighted;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyString;
    property OnKeyUp;
    property OnSelect;
    property OnStartDrag;
  end;

procedure Register;

implementation

uses
  Qt;

// contains the glyph for the control
{$R QColorBox.dcr}


procedure Register;
begin
  RegisterComponents('Samples', [TColorBox]);
end;

{ TColorBox }

procedure TColorBox.Change;
begin
  inherited;
  // when the user selects "Custom..."
  if FShowCustomColor and (ItemIndex = 0) then
  begin
    // display a color dialor for picking a color
    with TColorDialog.Create(self) do
    try
      Color := FCustomColor;
      if Execute then
        FCustomColor := Color;
    finally
      Free;
    end;
  end;
end;

procedure TColorBox.ColorCallBack(const ColorName: string);
begin
  // add every color name to the list 
  Items.Add(ColorName);
end;

constructor TColorBox.Create(AOwner: TComponent);
begin
  inherited;
  Style := csOwnerDrawFixed;
  PopulateItems;
end;

function TColorBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState): Boolean;
var
  BoxRect: TRect;
  TheColor: Longint;
begin
  Result := True;

  if not Focused then
    Canvas.Brush.Color := Color;
  Canvas.FillRect(Rect);
  if FShowCustomColor and (Index = 0) then
    Canvas.Brush.Color := FCustomColor
  else begin
    IdentToColor(Items[Index], TheColor);
    Canvas.Brush.Color := TColor(TheColor);
  end;

  InflateRect(Rect, -1, -1);
  BoxRect := Rect;
  BoxRect.Right := BoxRect.Bottom - BoxRect.Top;
  Canvas.FillRect(BoxRect);

  Canvas.Font.Assign(Font);

  if Index > -1 then
    Canvas.TextRect(Rect, BoxRect.Right + 1, Rect.Top, Items[Index]);
end;

function TColorBox.GetSelectedColor: TColor;
begin
  if FShowCustomColor and (ItemIndex = 0 ) then
    Result := FCustomColor
  else
    IdentToColor(Items[ItemIndex], Integer(Result));
end;

procedure TColorBox.Loaded;
begin
  inherited;
  PopulateItems;
end;

procedure TColorBox.PopulateItems;
begin
  Items.Clear;
  if FShowCustomColor then
    Items.Add('Custom...');
  GetColorValues(ColorCallBack);
end;

procedure TColorBox.SetCustomColor(const Value: TColor);
begin
  FCustomColor := Value;
  if FShowCustomColor and (ItemIndex = 0) then
    Invalidate;
end;

procedure TColorBox.SetShowCustomColor(const Value: Boolean);
begin
  if FShowCustomColor <> Value then
  begin
    FShowCustomColor := Value;
    PopulateItems;
  end;
end;

end.
